#ifndef DEMOB_CONN
#define DEMOB_CONN
#include "VisItControlInterface_V1.h"
#include <VisItDataInterface_V1.h>
#include "basin_funcs.h"
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <vector>
#include "SimHelperFunc.H"

void Update_UI_Commands();

struct pointMesh{
  Attribute *X;
  Attribute *Y;
  Attribute *Z;
  int numPoints;
};
struct gridMesh{
  double *min;
  double *max;
  int *nx;
};
struct Mesh{
  string *name;
  int type; //either 0 (VISIT_MESHTYPE_RECTILINEAR) or 3 (VISIT_MESHTYPE_POINT)
  int nDims;
  pointMesh *pm;
  gridMesh *gm;
};
struct demoVar{
  string name;
  vector<double> data;
};
struct demoVarB{
  string *name;
  Attribute *data;
  int size;
  char *meshName;
};

extern vector<demoVarB> visVarB;
extern vector<Mesh> meshVect;

VisIt_SimulationMetaData *VisItGetMetaData();
VisIt_MeshData *VisItGetMesh(int domain,const char *name);
VisIt_ScalarData *VisItGetScalar(int domain,const char *name);

//Overloaded function for creating a VisIt mesh from a list of points.
//These functions are used to plot data stored in a BASIN list.
//They are capable of exposing the list of points
//    both before and after VisIt has connected.
//IMPORTANT NOTE: These functions will not work if the name passed through
//             string &name has already been used for a mesh.
int makePointMesh(string &name, Attribute &x, Attribute &y, Attribute &z);
int makePointMesh(string &name, Attribute *x, Attribute *y, Attribute *z);
int makePointMesh(string &name, Attribute &x, Attribute &y);
int makePointMesh(string &name, Attribute *x, Attribute *y);

//Function for creating a VisIt mesh from gridded data.
//This function is used to plot 2 different types of data:
//    (1)data stored in a BASIN grid.
//    (2)list data that was moved onto a grid.
//        Example: data passed thru the function compute_density()
//It is capable of exposing the grid both before and after VisIt has connected.
//IMPORTANT NOTE: This function will not work if the name passed through
//             string &name has already been used for a mesh.
int makeGridMesh(string &name, vector<float> &ll, vector<float> &ur, vector<int> &nx);

//Overloaded function to expose data.
//These functions expose data to VisIt.
//They are capable of exposing data both before and after VisIt has connected.
//IMPORTANT NOTE: These functions will not work if the mesh given by
//            string &meshName has not been exposed to VisIt, or if
//            the name in string &name has already been given to
//            another variable.
int makeVariable(string &name, Attribute &data, string &meshName);
int makeVariable(string &name, Attribute *data, string &meshName);

#endif
